@@ -33,6 +33,7 @@ from django.contrib.admin.filterspecs import FilterSpec
 from django.contrib.admin.views.main import ChangeList, ERROR_FLAG
 from django.contrib.admin.options import IncorrectLookupParameters
 from django.contrib.admin.util import unquote, flatten_fieldsets
+from django.contrib.admin.models import LogEntry
 from django.contrib.sites.admin import Site, SiteAdmin
 from django.forms.models import ModelForm, BaseInlineFormSet, \
                                 fields_for_model, save_instance, modelformset_factory
@@ -312,6 +313,9 @@ class RelatedURLsModelAdmin(admin.ModelAdmin):
             url(r'^([^/]+)/delete/$',
                 wrap(self.delete_view),
                 name='%s_%s_delete' % info),
+            url(r'^([^/]+)/$',
+                wrap(self.change_view),
+                name='%s_%s_change' % info),
             url(r'^(.+)/$',
                 wrap(self.parse_path), )
         )
@@ -677,6 +681,7 @@ class WorkflowBatchActionProvider(object):
         selected = request.POST.getlist(admin.ACTION_CHECKBOX_NAME)
         if selected:
             if request.POST.get('post', False):
+                original_status_dict = dict(queryset.values_list('pk', 'status'))
                 updated = queryset.update(status=state)
                 obj_log = ugettext("Changed to %s") % state
                 msg_data = {'number': updated,
@@ -684,6 +689,7 @@ class WorkflowBatchActionProvider(object):
                             'state': state}
                 msg = ugettext(u"Successfully set %(number)d %(model_name)s as %(state)s.") % msg_data
                 for obj in queryset:
+                    obj._original_status = original_status_dict.get(obj.pk, obj.status)
                     self.log_change(request, obj, obj_log)
                     obj.save()
                 self.message_user(request, msg)
@@ -691,7 +697,7 @@ class WorkflowBatchActionProvider(object):
                 extra_context = {'title': confirm_msg,
                                  'action_submit': 'set_as_%s' % state}
                 return self.confirm_action(request, queryset, extra_context)
-    change_state.short_description = _(u"Change state of selected % (verbose_name_plural)s")
+    change_state.short_description = _(u"Change state of selected %(verbose_name_plural)s")
 
 
 class StatusControlProvider(object):
@@ -743,6 +749,11 @@ class BaseContentAdmin(BaseAdmin, WorkflowBatchActionProvider, StatusControlProv
                                     'size': 100}, }
     exclude = ('adquire_global_permissions', )
 
+    def __init__(self, *args, **kwargs):
+        super(BaseContentAdmin, self).__init__(*args, **kwargs)
+        # Save original prepopulated fields just in case we have to remove any readonly field from it
+        self.original_prepopulated_fields = self.prepopulated_fields.copy()
+
     def get_urls(self):
         from django.conf.urls.defaults import patterns
         urls = super(BaseContentAdmin, self).get_urls()
@@ -1348,6 +1359,89 @@ class AnnouncementAdmin(AnnouncementDefaultAdmin):
     form = AnnouncementAdminForm
 
 
+class LogEntryRelatedContentModelAdmin(admin.ModelAdmin):
+    change_list_template = "admin/logentry/changelog.html"
+    list_display = ('logentry_link',
+                    'get_link_public_url',
+                    'get_link_contenttype',
+                    'get_culpright',
+                    'get_action', 'get_link_admin_url',)
+    date_hierarchy = 'action_time'
+    list_filter = ('content_type', 'user')
+    actions = None
+
+    def get_url(self, logentry, admin=False, url=None, label=None):
+        if not logentry.object_id.isdigit():
+            return _(u'Error in id')
+        if logentry.object_id and not url:
+            try:
+                obj = logentry.content_type.model_class().objects.get(pk=logentry.object_id)
+            except models.ObjectDoesNotExist:
+                return label or _(u'deleted')
+            if admin:
+                url = '/admin/%s' % logentry.get_admin_url()
+            else:
+                get_absolute_url = getattr(obj, 'get_absolute_url', '')
+                url = get_absolute_url and get_absolute_url() or get_absolute_url
+            label = label or url
+
+        if url:
+            if len(label) > 30:
+                label = "%s ..." % label[:30]
+            return mark_safe("<a href='%s'>%s</a>" % (url, label))
+        elif label:
+            return label
+        return '---'
+
+    # COLUMNS
+
+    def logentry_link(self, logentry):
+        return mark_safe('<a id="logentry%s">%s</a>' % (logentry.id, logentry.action_time))
+    logentry_link.allow_tags = True
+    logentry_link.short_description = _(u'Log entry')
+
+    def get_culpright(self, logentry):
+        user = logentry.user
+        return mark_safe(u'<a href="/admin/auth/user/%s/">%s</a>' % (user.id, user.get_full_name() or user.username))
+    get_culpright.allow_tags = True
+    get_culpright.short_description = _(u'User')
+
+    def get_link_admin_url(self, logentry):
+        return self.get_url(logentry, admin=True)
+    get_link_admin_url.allow_tags = True
+    get_link_admin_url.short_description = _(u'Admin url')
+
+    def get_link_public_url(self, logentry):
+        if len(logentry.object_repr) < 40:
+            label = logentry.object_repr
+        else:
+            label = "%s..." % logentry.object_repr[:37]
+        return self.get_url(logentry, admin=False, label=label)
+    get_link_public_url.allow_tags = True
+    get_link_public_url.short_description = _(u'Public url')
+
+    def get_link_contenttype(self, logentry):
+        model_class = logentry.content_type.model_class()
+        return self.get_url(logentry,
+                            url='/admin/%s/%s/' %
+                               (model_class._meta.app_label,
+                                model_class._meta.module_name),
+                            label=logentry.content_type.__unicode__())
+    get_link_contenttype.allow_tags = True
+    get_link_contenttype.short_description = _(u'Content type')
+
+    def get_action(self, logentry):
+        if logentry.is_addition():
+            return _(u'Added')
+        elif logentry.is_deletion():
+            return _(u'Deleted')
+        elif logentry.is_change():
+            return logentry.change_message
+        return '---'
+    get_action.allow_tags = True
+    get_action.short_description = _(u'Action')
+
+
 def register(site):
     ## register admin models
     site.register(BaseContent, BaseContentViewAdmin)
@@ -1355,6 +1449,7 @@ def register(site):
     site.register(ProviderRule)
     site.register(StoredOEmbed)
     site.register(Announcement, AnnouncementAdmin)
+    site.register(LogEntry, LogEntryRelatedContentModelAdmin)
 
     #default notification
     site.register(NoticeType, NoticeTypeAdmin)
@@ -1370,7 +1465,6 @@ def register_related_base(site, related_to):
     site.register_related(ContactInfo, BaseContentRelatedContactInfoAdmin, related_to=related_to)
     site.register_related(BaseContent, PermissionRelatedAdmin, related_to=related_to)
 
-
 # ----- begin monkey patching -----
 
 # we change ChangeList.get_ordering for allowing define a dynamic ordering
