@@ -1,5 +1,9 @@
-from copy import deepcopy
 from django.conf import settings
+from django.template.loader import render_to_string
+from django.template import RequestContext
+
+from copy import deepcopy
+from django.db import connection
 from south.db import db
 from transmeta import (get_real_fieldname_in_each_language, get_field_language,
                        fallback_language)
@@ -32,4 +36,15 @@ def add_south_trans_column(table, model_name, field_name, orm):
 
 def delete_south_trans_column(table, field_name):
     for real_field in get_real_fieldname_in_each_language(field_name):
-        db.delete_column(table, real_field)
\ No newline at end of file
+        db.delete_column(table, real_field)
+
+
+def table_exists(table):
+    return table in connection.introspection.get_table_list(connection.cursor())
+
+
+def get_render_http_error(request, http_error):
+    template = settings.HTTP_STATUS_CODE_TEMPLATES.get(http_error, None)
+    context = {'template_base': 'viewlet_error.html', 'is_admin': False}
+    return render_to_string(template, context,
+                                      context_instance=RequestContext(request))
