@@ -17,6 +17,7 @@
 
 from django.conf import settings
 from django.contrib import admin
+from django.db.models import Q
 from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext_lazy as _
 from django.utils.translation import ugettext
@@ -29,15 +30,16 @@ from merengue.base.admin import set_field_read_only
 from merengue.base.widgets import RelatedFieldWidgetWrapperWithoutAdding
 from merengue.section.fields import CSSValidatorField
 from merengue.section.forms import MenuAdminModelForm
-from merengue.section.models import (Menu, Section,
+from merengue.section.models import (Menu, BaseSection,
                                      BaseLink, AbsoluteLink, ContentLink, ViewletLink,
                                      Document, DocumentSection, CustomStyle,
                                      SectionRelatedContent, CustomStyleImage)
 from merengue.section.formsets import BaseLinkInlineFormSet
 from merengue.perms import utils as perms_api
+from merengue.perms.admin import PermissionAdmin
 
 
-class BaseSectionAdmin(BaseOrderableAdmin):
+class BaseSectionAdmin(BaseOrderableAdmin, PermissionAdmin):
     sortablefield = 'order'
     ordering = ('order', )
     list_display = ('name', 'slug', )
@@ -56,13 +58,27 @@ class BaseSectionAdmin(BaseOrderableAdmin):
         """
         Overrides Django admin behaviour to add ownership based access control
         """
-        return self.has_add_permission(request)
+        if perms_api.has_global_permission(request.user, 'manage_section'):
+            return True
+        else:
+            if BaseSection.objects.filter(owners=request.user).count():
+                return True
+        return False
 
     def has_delete_permission(self, request, obj=None):
         """
         Overrides Django admin behaviour to add ownership based access control
         """
-        return self.has_add_permission(request)
+        return perms_api.has_global_permission(request.user, 'manage_section')
+
+    def queryset(self, request):
+        qs = super(BaseSectionAdmin, self).queryset(request)
+        if perms_api.has_global_permission(request.user, 'manage_portal'):
+            return qs
+        elif self.has_change_permission(request):
+            return qs.filter(Q(owners=request.user))
+        else:
+            return qs.none()
 
     def get_form(self, request, obj=None, **kwargs):
         form = super(BaseSectionAdmin, self).get_form(request, obj, **kwargs)
@@ -72,7 +88,7 @@ class BaseSectionAdmin(BaseOrderableAdmin):
             if not obj:
                 qs = qs.model.objects.get_empty_query_set()
             else:
-                qs = qs.filter(basesection=obj)
+                qs = qs.filter(sections=obj)
 
             # Como Document no esta registrado en el admin site, no tiene
             # sentido mostrar este campo si no tiene opciones ya que no
@@ -121,8 +137,9 @@ class SectionAdmin(BaseSectionAdmin):
 
 
 class SectionContentAdmin(OrderableRelatedModelAdmin):
-    related_field = 'basesection'
+    related_field = 'sections'
     sortablefield = 'order'
+    manage_contents = True
 
     def custom_relate_content(self, request, obj, form, change):
         if not change:
@@ -249,7 +266,7 @@ class ContentLinkInline(BaseLinkInline):
             qs = formfield.queryset
             admin_model = getattr(self, 'admin_model', None)
             if admin_model:
-                qs = qs.filter(basesection=admin_model.basecontent)
+                qs = qs.filter(sections=admin_model.basecontent)
             formfield.queryset = qs
             # change the widget to a wrapper
             # note: formfield.wigdet is already a wrapper of formfield.widget.widget
@@ -405,6 +422,10 @@ class MainMenuRelatedAdmin(BaseSectionMenuRelatedAdmin):
     tool_label = _('main menu')
     related_field = 'main_menu_section'
 
+    def changelist_view(self, request, extra_context={}, parent_model_admin=None, parent_object=None):
+        extra_context = self._update_extra_context(request, extra_context, parent_model_admin, parent_object)
+        return super(MainMenuRelatedAdmin, self).changelist_view(request, extra_context)
+
 
 class PortalMenuAdmin(MenuAdmin):
     tool_name = 'portalmenu'
@@ -443,14 +464,14 @@ class DocumentSectionRelatedModelAdmin(RelatedModelAdmin):
 
 
 def register_related(site):
-    site.register_related(Document, DocumentRelatedModelAdmin, related_to=Section)
-    site.register_related(CustomStyle, CustomStyleRelatedModelAdmin, related_to=Section)
-    site.register_related(Menu, MainMenuRelatedAdmin, related_to=Section)
+    site.register_related(Document, DocumentRelatedModelAdmin, related_to=BaseSection)
+    site.register_related(CustomStyle, CustomStyleRelatedModelAdmin, related_to=BaseSection)
+    site.register_related(Menu, MainMenuRelatedAdmin, related_to=BaseSection)
     site.register_related(DocumentSection, DocumentSectionRelatedModelAdmin, related_to=Document)
 
 
 def register(site):
-    site.register(Section, SectionAdmin)
+    site.register(BaseSection, SectionAdmin)
     site.register(Document, DocumentAdmin)
     site.register(Menu, PortalMenuAdmin)
     register_related(site)
