@@ -1,4 +1,4 @@
-# Copyright (c) 2010 by Yaco Sistemas <msaelices@yaco.es>
+# Copyright (c) 2010 by Yaco Sistemas
 #
 # This file is part of Merengue.
 #
@@ -16,11 +16,9 @@
 # along with Merengue.  If not, see <http://www.gnu.org/licenses/>.
 
 from django import template
-from django.conf import settings
-from django.template import RequestContext
-from django.template.loader import render_to_string
 from django.http import Http404
 from django.core.exceptions import PermissionDenied
+from merengue.base.utils import get_render_http_error
 
 register = template.Library()
 
@@ -34,10 +32,10 @@ class RenderViewletNode(template.Node):
         request = context.get('request', None)
         try:
             if self.registered_viewlet is not None:
-                viewlet = self.registered_viewlet.resolve(context).get_registry_item_class()
-            else:
-                viewlet = None
-            return viewlet.render(request)
+                viewlet = self.registered_viewlet.resolve(context).get_registry_item()
+            else:  # if no viewlet found
+                return ''
+            return viewlet.render(request, context)
         except template.VariableDoesNotExist:
             return ''
         except PermissionDenied:
@@ -58,13 +56,3 @@ def do_render_viewlet(parser, token):
         raise template.TemplateSyntaxError('"%r" tag requires at one argument' % tag_name)
     registered_viewlet = parser.compile_filter(bits[1])
     return RenderViewletNode(registered_viewlet)
-
-
-# ----- auxiliar functions ----- #
-
-
-def get_render_http_error(request, http_error):
-    template = settings.HTTP_STATUS_CODE_TEMPLATES.get(http_error, None)
-    context = {'template_base': 'viewlet_error.html', 'is_admin': False}
-    return render_to_string(template, context,
-                                      context_instance=RequestContext(request))
